/********************************************************************
 KWin - the KDE window manager
 This file is part of the KDE project.

Copyright (C) 2018 Vlad Zahorodnii <vlad.zahorodnii@kde.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/

"use strict";

var fadingDropdownMenuEffect = {
    loadConfig: function () {
        fadingDropdownMenuEffect.fadeInDuration = animationTime(200);
        fadingDropdownMenuEffect.fadeOutDuration = animationTime(150);
    },
    slotWindowAdded: function (window) {
        if (!window.dropdownMenu && !window.comboBox) {
            return;
        }
        window.fadeInAnimation = animate({
            window: window,
            curve: Effect.WindowAddedOpacityCurve,
            duration: fadingDropdownMenuEffect.fadeInDuration,
            animations: [
                {
                    type: Effect.Opacity,
                    from: 0.0,
                    to: 1.0
                },
                {
                    type: Effect.Position,
                    from: {
                        value1: window.geometry.x + window.geometry.width / 2,
                        value2: window.geometry.y + window.geometry.height / 2 - 16
                    },
                    to: {
                        value1: window.geometry.x + window.geometry.width / 2,
                        value2: window.geometry.y + window.geometry.height / 2
                    }
                }
            ]
        });
    },
    slotWindowClosed: function (window) {
        if (!window.dropdownMenu && !window.comboBox) {
            return;
        }
        window.fadeOutAnimation = animate({
            window: window,
            curve: Effect.WindowClosedOpacityCurve,
            duration: fadingDropdownMenuEffect.fadeOutDuration,
            animations: [
                {
                    type: Effect.Opacity,
                    from: 1.0,
                    to: 0.0
                },
                {
                    type: Effect.Position,
                    from: {
                        value1: window.geometry.x + window.geometry.width / 2,
                        value2: window.geometry.y + window.geometry.height / 2
                    },
                    to: {
                        value1: window.geometry.x + window.geometry.width / 2,
                        value2: window.geometry.y + window.geometry.height / 2 - 16
                    }
                }
            ]
        });
    },
    slotWindowDataChanged: function (window, role) {
        if (role == Effect.WindowAddedGrabRole) {
            if (window.fadeInAnimation && effect.isGrabbed(window, role)) {
                cancel(window.fadeInAnimation);
                delete window.fadeInAnimation;
            }
        } else if (role == Effect.WindowClosedGrabRole) {
            if (window.fadeOutAnimation && effect.isGrabbed(window, role)) {
                cancel(window.fadeOutAnimation);
                delete window.fadeOutAnimation;
            }
        }
    },
    init: function () {
        fadingDropdownMenuEffect.loadConfig();

        effect.configChanged.connect(fadingDropdownMenuEffect.loadConfig);
        effects.windowAdded.connect(fadingDropdownMenuEffect.slotWindowAdded);
        effects.windowClosed.connect(fadingDropdownMenuEffect.slotWindowClosed);
        effects.windowDataChanged.connect(fadingDropdownMenuEffect.slotWindowDataChanged);
    }
};

fadingDropdownMenuEffect.init();
